/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(d7e1364c-61a0-4cf1-bae3-a960e87064e1)]
interface nsIMobileNetworkInfo: nsISupports
{
  /**
   * Short name of the network operator.
   */
  readonly attribute DOMString shortName;

  /**
   * Long name of the network operator.
   */
  readonly attribute DOMString longName;

  /**
   * Mobile Country Code (MCC) of the network operator.
   */
  readonly attribute DOMString mcc;

  /**
   * Mobile Network Code (MNC) of the network operator.
   */
  readonly attribute DOMString mnc;

  /**
   * State of this network operator.
   *
   * Possible values: 'available', 'connected', 'forbidden', or null (unknown).
   */
  readonly attribute DOMString state;
};
