/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsINetworkProperties;

[uuid(7956523b-631e-4f80-94a5-3883bcfd6bf3)]
interface nsIMozNavigatorNetwork : nsISupports
{
  readonly attribute nsINetworkProperties properties;
};
