/* THIS FILE IS AUTOGENERATED FROM MediaKeysRequestStatus.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_MediaKeysRequestStatusBinding_h
#define mozilla_dom_MediaKeysRequestStatusBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct NativePropertyHooks;
class ProtoAndIfaceCache;
struct RequestMediaKeySystemAccessNotificationAtoms;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

enum class MediaKeySystemStatus : uint32_t {
  Available,
  Api_disabled,
  Cdm_disabled,
  Cdm_not_supported,
  Cdm_not_installed,
  Cdm_insufficient_version,
  Cdm_created,
  Error,
  EndGuard_
};

namespace MediaKeySystemStatusValues {
extern const EnumEntry strings[9];
} // namespace MediaKeySystemStatusValues


struct RequestMediaKeySystemAccessNotification : public DictionaryBase
{
  nsString mKeySystem;
  MediaKeySystemStatus mStatus;

  RequestMediaKeySystemAccessNotification();

  explicit inline RequestMediaKeySystemAccessNotification(const FastDictionaryInitializer& )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline RequestMediaKeySystemAccessNotification(const RequestMediaKeySystemAccessNotification& aOther)
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const RequestMediaKeySystemAccessNotification& aOther);

private:
  static bool
  InitIds(JSContext* cx, RequestMediaKeySystemAccessNotificationAtoms* atomsCache);
};

namespace binding_detail {
struct FastRequestMediaKeySystemAccessNotification : public RequestMediaKeySystemAccessNotification
{
  inline FastRequestMediaKeySystemAccessNotification()
    : RequestMediaKeySystemAccessNotification(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MediaKeysRequestStatusBinding_h
