GENERATED_EVENT(AutocompleteErrorEvent)
GENERATED_EVENT(BlobEvent)
GENERATED_EVENT(CFStateChangeEvent)
GENERATED_EVENT(CSSFontFaceLoadEvent)
GENERATED_EVENT(CallEvent)
GENERATED_EVENT(CallGroupErrorEvent)
GENERATED_EVENT(CameraClosedEvent)
GENERATED_EVENT(CameraConfigurationEvent)
GENERATED_EVENT(CameraFacesDetectedEvent)
GENERATED_EVENT(CameraStateChangeEvent)
GENERATED_EVENT(CaretStateChangedEvent)
GENERATED_EVENT(CloseEvent)
GENERATED_EVENT(DOMTransactionEvent)
GENERATED_EVENT(DataErrorEvent)
GENERATED_EVENT(DataStoreChangeEvent)
GENERATED_EVENT(DeviceLightEvent)
GENERATED_EVENT(DeviceOrientationEvent)
GENERATED_EVENT(DeviceProximityEvent)
GENERATED_EVENT(DeviceStorageAreaChangedEvent)
GENERATED_EVENT(DeviceStorageChangeEvent)
GENERATED_EVENT(DownloadEvent)
GENERATED_EVENT(ErrorEvent)
GENERATED_EVENT(GamepadAxisMoveEvent)
GENERATED_EVENT(GamepadButtonEvent)
GENERATED_EVENT(GamepadEvent)
GENERATED_EVENT(HashChangeEvent)
GENERATED_EVENT(IccChangeEvent)
GENERATED_EVENT(ImageCaptureErrorEvent)
GENERATED_EVENT(MediaStreamEvent)
GENERATED_EVENT(MediaStreamTrackEvent)
GENERATED_EVENT(MozApplicationEvent)
GENERATED_EVENT(MozCellBroadcastEvent)
GENERATED_EVENT(MozClirModeEvent)
GENERATED_EVENT(MozContactChangeEvent)
GENERATED_EVENT(MozEmergencyCbModeEvent)
GENERATED_EVENT(MozInterAppMessageEvent)
GENERATED_EVENT(MozMessageDeletedEvent)
GENERATED_EVENT(MozMmsEvent)
GENERATED_EVENT(MozOtaStatusEvent)
GENERATED_EVENT(MozSettingsEvent)
GENERATED_EVENT(MozSettingsTransactionEvent)
GENERATED_EVENT(MozSmsEvent)
GENERATED_EVENT(MozStkCommandEvent)
GENERATED_EVENT(MozVoicemailEvent)
GENERATED_EVENT(PageTransitionEvent)
GENERATED_EVENT(PluginCrashedEvent)
GENERATED_EVENT(PopStateEvent)
GENERATED_EVENT(PopupBlockedEvent)
GENERATED_EVENT(ProgressEvent)
GENERATED_EVENT(RTCDataChannelEvent)
GENERATED_EVENT(RTCPeerConnectionIceEvent)
GENERATED_EVENT(RecordErrorEvent)
GENERATED_EVENT(ScrollViewChangeEvent)
GENERATED_EVENT(SelectionStateChangedEvent)
GENERATED_EVENT(SpeechRecognitionEvent)
GENERATED_EVENT(SpeechSynthesisEvent)
GENERATED_EVENT(StyleRuleChangeEvent)
GENERATED_EVENT(StyleSheetApplicableStateChangeEvent)
GENERATED_EVENT(StyleSheetChangeEvent)
GENERATED_EVENT(TVCurrentChannelChangedEvent)
GENERATED_EVENT(TVCurrentSourceChangedEvent)
GENERATED_EVENT(TVEITBroadcastedEvent)
GENERATED_EVENT(TVScanningStateChangedEvent)
GENERATED_EVENT(TrackEvent)
GENERATED_EVENT(UDPMessageEvent)
GENERATED_EVENT(USSDReceivedEvent)
GENERATED_EVENT(UserProximityEvent)
